unit ahDBImage;

interface

uses
{$IFDEF VER140}   { Delphi 6 }
  Variants,
{$ENDIF}
{$IFDEF VER150}   { Delphi 7 }
  Variants,
{$ENDIF}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DBCtrls, DB, JPEG;

type
  // q~ɮרo

  TahImageThread = class(TThread)
  private
    { Private declarations }
    FComponent: TComponent;
  protected
    { Protected declarations }
    procedure DrawPicture;
    procedure Execute; override;
  public
    { Public declarations }
    constructor Create(AComponent: TComponent);
  published
    { Published declarations }
  end;


  TahDBImage = class(TImage)
  private
    { Private declarations }
    FDataField,
    FHostDriver: String;
    FDataLink: TFieldDataLink;
    FActivePicture: Boolean;
    FThread: TahImageThread;

    function GetDataSource: TDataSource;
    procedure SetDataSource(const Value: TDataSource);
    procedure SetDataField(const Value: String);
    procedure SetActivePicture(const Value: Boolean);

    procedure NewDataChange(Sender: TObject);
    procedure NewActiveChange(Sender: TObject);
  protected
    { Protected declarations }
    procedure DrawPicture(Sender: TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure JpegLoadFromFile(const Filename: string);
    procedure JpegLoadFromStream(Stream: TStream);
  published
    { Published declarations }
    property HostDriver: String read FHostDriver write FHostDriver;
    property DataField: String read FDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ActivePicture: Boolean read FActivePicture write SetActivePicture default True;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TahDBImage]);
end;

{ TahDBImage }

constructor TahDBImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := NewDataChange;
  FDataLink.OnActiveChange := NewActiveChange;

  FActivePicture := True;
  FThread := nil;
end;

destructor TahDBImage.Destroy;
begin
  if Assigned(FThread) then
    FThread.Terminate;
  Picture.Bitmap := nil;
  
  FDataLink.OnDataChange := nil;
  FDataLink.OnActiveChange := nil;
  FreeAndNil(FDataLink);

  inherited Destroy;
end;

function TahDBImage.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TahDBImage.JpegLoadFromFile(const Filename: string);
begin
  if FileExists(Filename) then
    Picture.LoadFromFile(Filename);
end;

procedure TahDBImage.JpegLoadFromStream(Stream: TStream);
begin
  if (Stream.Size > 0) then
  begin
    Stream.Position := 0;
    Picture.Bitmap.LoadFromStream(Stream);
  end;  
end;

procedure TahDBImage.NewDataChange(Sender: TObject);
begin
  if not (csDesigning in ComponentState) then
  begin
    if Assigned(FThread) then
      FThread.Terminate;

    FThread := TahImageThread.Create(Self);
  end;
end;

procedure TahDBImage.SetDataField(const Value: String);
begin
  FDataField := Value;
  FDataLink.FieldName := Value;
  if Assigned(FDataLink) then
    if not Assigned(FDataLink.Field) then
      Picture.Bitmap := nil;
end;

procedure TahDBImage.SetDataSource(const Value: TDataSource);
begin
  if (FDataLink.DataSource <> Value) then
    FDataLink.DataSource := Value;
end;

procedure TahDBImage.SetActivePicture(const Value: Boolean);
begin
  if (FActivePicture <> Value) then
  begin
    FActivePicture := Value;
    if not FActivePicture then
    begin
      if Assigned(FThread) then
        FThread.Terminate;
    end;

    NewDataChange(Self);
  end;  
end;

procedure TahDBImage.NewActiveChange(Sender: TObject);
begin
  NewDataChange(Self);
end;

procedure TahDBImage.DrawPicture(Sender: TObject);
var
  strPictFileName: String;
  booNotEmpty: Boolean;
begin
  booNotEmpty := False;
  try
    if (FDataLink <> nil) then
    begin
      if (FDataLink.Field <> nil) and FDataLink.Active then
      begin
        if FActivePicture then
        begin
          strPictFileName := Trim(FDataLink.Field.AsString);
          if (strPictFileName <> '') then
          begin
            strPictFileName := HostDriver + '\' + strPictFileName;
            if FileExists(strPictFileName) then
            begin
              booNotEmpty := True;
              Picture.LoadFromFile(strPictFileName);
            end;
          end;
        end;
      end;
    end;

    if not booNotEmpty then
      Picture.Bitmap := nil;
  except

  end;
end;

{ TahImageThread }

constructor TahImageThread.Create(AComponent: TComponent);
begin
  inherited Create(False); // إ߫ߨ
  FComponent := AComponent;
  Priority := tpLowest;
end;

procedure TahImageThread.DrawPicture;
begin
  with TahDBImage(FComponent) do
  begin
    DrawPicture(FComponent);
  end;
end;

procedure TahImageThread.Execute;
begin
  FreeOnTerminate := True;
  while (not Terminated) do
  begin
    if Terminated then
      Break;
    Synchronize(DrawPicture);
    SleepEx(1000, False);
  end;
end;

end.
